@echo off
setlocal enabledelayedexpansion

rem Check for the SDK.
if not defined OUTRUN_SDK_PATH ( 
  rem Backup plan: check whether we can find setupenv.bat ourselves, and use it for the time being.
  if exist "%~dp0\..\setupenv.bat" ( 
    call %~dp0\..\setupenv.bat
    if errorlevel 1 goto error
  ) else (
    echo OUTRUN_SDK_PATH environment variable not set. Please run setupenv.bat!
    exit /b 1
  )
)

rem Set GCC version, if not defined externally.
if not defined OUTRUN_GCC_VERSION (
  set OUTRUN_GCC_VERSION=8.2.0
)

set OUTRUN_SDK_INCLUDE=%OUTRUN_SDK_PATH%/include
set OUTRUN_SDK_LDSCRIPT=%OUTRUN_SDK_PATH%/ldscript
set OUTRUN_SDK_LIB=%OUTRUN_SDK_PATH%/lib
set OUTPUT_PATH=output

set clean=
if "%1"=="clean" set clean=1

if not defined OUTRUN_GCC_PATH ( 
  echo OUTRUN_GCC_PATH environment variable not set. Please run setenv.bat!
  exit /b 1
)

set OUTRUN_GCC_ARCH=m68k-elf
set OUTRUN_GCC_PREFIX=!OUTRUN_GCC_ARCH!-
set OUTRUN_GCC_CPUTYPE=m68000

rem set CC_OPTS=

rem CPU type subpath. If we have multi-libs (pre GCC 8), we need to specify /m68000 to pick the correct startup/libraries.
set CPU_PATH=
if exist !OUTRUN_GCC_PATH!\lib\gcc\!OUTRUN_GCC_ARCH!\!OUTRUN_GCC_VERSION!\!OUTRUN_GCC_CPUTYPE! (
  set CPU_PATH=/!OUTRUN_GCC_CPUTYPE!
)

if not exist !OUTPUT_PATH! mkdir !OUTPUT_PATH!

rem clean out old linker scripts.
if exist "!OUTPUT_PATH!\main.link.in" del "!OUTPUT_PATH!\main.link.in"
if exist "!OUTPUT_PATH!\sub.link.in" del "!OUTPUT_PATH!\sub.link.in"

rem build file lists.
set inputfiles0=
set inputfiles1=
set sharedfiles=
set any_files=0
for %%i in (*.c *.cpp *.s ..\common\*.c) do (
  set inputfile=%%i
  set any_files=1
  
  rem test for *sub* and *shared*, and put files in the appropriate lists.
  set substr=!inputfile:sub=!
  if not "x!substr!"=="x!inputfile!" (
    set inputfiles1=!inputfiles1! !inputfile!    
  ) else (
	  set sharedstr=!inputfile:shared=!
	  if not "x!sharedstr!"=="x!inputfile!" (
	    set sharedfiles=!sharedfiles! !inputfile!
	  ) else (
	    set inputfiles0=!inputfiles0! !inputfile!
	  )
  )
)

if !any_files! equ 0 (
  echo No input files found. Nothing to do.
  exit /b 1
)

rem Determine what CPUs to build for. Always build CPU 0.
set cpus=0 1
if "!inputfiles1!"=="" set cpus=0

rem compile our files.
if !clean! neq 1 (
  echo Compiling using GCC version %OUTRUN_GCC_VERSION%...
  if not "!CPU_PATH!"=="" echo Detected multi-lib; using CPU type '!OUTRUN_GCC_CPUTYPE!'
)

rem for every cpu, combine shared and cpu-specific, and build all files in it.
for %%c in (!cpus!) do (
  set inputlists=inputfiles%%c sharedfiles

  set cpudef=CPU%%c
  set shared=  
  for %%l in (!inputlists!) do (
    rem echo list %%l: !%%l!
	if "%%l" == "sharedfiles" set shared=1
	
	rem build all files in this list.
	for %%i in (!%%l!) do (
	  
	  rem determine output name.
	  set outputfile=%%~ni.o
	  if "!shared!"=="1" set outputfile=%%~ni_cpu%%c.o
	  
	  if !clean! equ 1 (
  	    rem clean.
	    if exist "!OUTPUT_PATH!\!outputfile!" del "!OUTPUT_PATH!\!outputfile!"
	  ) else (
		rem compile.
		echo %%~ni%%~xi
		  
		if %%~xi? == .c? (
		  %OUTRUN_GCC_PREFIX%gcc -c %%i -std=gnu11 -m68000 -o !OUTPUT_PATH!/!outputfile! -Os -D!CPUDEF! -I. -I!OUTRUN_SDK_INCLUDE! -I!OUTRUN_SDK_INCLUDE!\!cpudef! -I../common
		)
		if %%~xi? == .cpp? (
		  %OUTRUN_GCC_PREFIX%g++ -c %%i --no-rtti -m68000 -o !OUTPUT_PATH!/!outputfile! -Os -D!CPUDEF! -I. -I!OUTRUN_SDK_INCLUDE! -I!OUTRUN_SDK_INCLUDE!\!cpudef! -I../common
		)
		if %%~xi? == .s? (
		  %OUTRUN_GCC_PREFIX%as %%i -m68000 -o !OUTPUT_PATH!/!outputfile! --defsym !CPUDEF!=1
		)

		if ERRORLEVEL 1 goto error
		
		rem append to linker input lists
		if %%c equ 1 (
		  echo !OUTPUT_PATH!/!outputfile! >> "!OUTPUT_PATH!\sub.link.in"
		) else (
		  echo !OUTPUT_PATH!/!outputfile! >> "!OUTPUT_PATH!\main.link.in"
        )
      )
	)
  )
)

if !clean! equ 1 goto clean

rem link
echo Linking...

rem Linker settings.
set "LD_CRT_PATH=!OUTRUN_GCC_PATH!/lib/gcc/!OUTRUN_GCC_ARCH!/!OUTRUN_GCC_VERSION!!CPU_PATH!"
set "LD_LIBS=-lc -lgcc"
set "LD_LIBS0=!OUTRUN_SDK_LIB!/outrun_sdk_cpu0.lib !LD_LIBS!"
set "LD_LIBS1=!OUTRUN_SDK_LIB!/outrun_sdk_cpu1.lib !LD_LIBS!"
set LD_DIRS=-L"%OUTRUN_GCC_PATH%/lib/gcc/!OUTRUN_GCC_ARCH!/%OUTRUN_GCC_VERSION%%CPU_PATH%" -L"%OUTRUN_GCC_PATH%/!OUTRUN_GCC_ARCH!/lib%CPU_PATH%"

echo maincpu_rom.bin
rem crti.o crtbegin.o ... -lgcc crtend.o crtn.o
%OUTRUN_GCC_PREFIX%ld "!LD_CRT_PATH!/crtbegin.o" @!OUTPUT_PATH!/main.link.in !LD_LIBS0! "!LD_CRT_PATH!/crtend.o" !LD_DIRS! --script=%OUTRUN_SDK_LDSCRIPT%/outrun_main_rom.ld -o !OUTPUT_PATH!/maincpu_rom.bin --Map=!OUTPUT_PATH!/maincpu_rom.map
if ERRORLEVEL 1 goto error
echo maincpu_ram.bin
%OUTRUN_GCC_PREFIX%ld "!LD_CRT_PATH!/crtbegin.o" @!OUTPUT_PATH!/main.link.in !LD_LIBS0! "!LD_CRT_PATH!/crtend.o" !LD_DIRS! --script=%OUTRUN_SDK_LDSCRIPT%/outrun_main_ram.ld -o !OUTPUT_PATH!/maincpu_ram.bin --Map=!OUTPUT_PATH!/maincpu_ram.map
if ERRORLEVEL 1 goto error

if exist "!OUTPUT_PATH!\sub.link.in" (
  echo subcpu_rom.bin
  %OUTRUN_GCC_PREFIX%ld "!LD_CRT_PATH!/crtbegin.o" @!OUTPUT_PATH!/sub.link.in !LD_LIBS1! "!LD_CRT_PATH!/crtend.o" !LD_DIRS! --script=%OUTRUN_SDK_LDSCRIPT%/outrun_sub_rom.ld -o !OUTPUT_PATH!/subcpu_rom.bin --Map=!OUTPUT_PATH!/subcpu_rom.map
  if ERRORLEVEL 1 goto error
  echo subcpu_ram.bin
  %OUTRUN_GCC_PREFIX%ld "!LD_CRT_PATH!/crtbegin.o" @!OUTPUT_PATH!/sub.link.in !LD_LIBS1! "!LD_CRT_PATH!/crtend.o" !LD_DIRS! --script=%OUTRUN_SDK_LDSCRIPT%/outrun_sub_ram.ld -o !OUTPUT_PATH!/subcpu_ram.bin --Map=!OUTPUT_PATH!/subcpu_ram.map
  if ERRORLEVEL 1 goto error
)

rem delete linker input lists
if exist "!OUTPUT_PATH!\main.link.in" del "!OUTPUT_PATH!\main.link.in"
if exist "!OUTPUT_PATH!\sub.link.in" del "!OUTPUT_PATH!\sub.link.in"

rem build rom images.
echo Building ROM images...

splitbin.exe "!OUTPUT_PATH!\maincpu_rom.bin" 65536 2 "!OUTPUT_PATH!\epr-10380b.133" "!OUTPUT_PATH!\epr-10382b.118" "!OUTPUT_PATH!\epr-10381b.132" "!OUTPUT_PATH!\epr-10383b.117"
if ERRORLEVEL 1 goto error

if exist "!OUTPUT_PATH!\subcpu_rom.bin" (
  splitbin.exe "!OUTPUT_PATH!\subcpu_rom.bin" 65536 2 "!OUTPUT_PATH!\epr-10327a.76" "!OUTPUT_PATH!\epr-10329a.58" "!OUTPUT_PATH!\epr-10328a.75" "!OUTPUT_PATH!\epr-10330a.57"
  if ERRORLEVEL 1 goto error
)

echo Done.
goto end

:clean
rem object files already deleted.

rem rom files
for %%i in (epr-10380b.133 epr-10382b.118 epr-10381b.132 epr-10383b.117) do (
  if exist "!OUTPUT_PATH!\%%i" del "!OUTPUT_PATH!\%%i"
)

for %%i in (epr-10327a.76 epr-10329a.58 epr-10328a.75 epr-10330a.57) do (
  if exist "!OUTPUT_PATH!\%%i" del "!OUTPUT_PATH!\%%i"
)

for %%i in (main.link.in sub.link.in maincpu_ram.bin maincpu_ram.map maincpu_rom.bin maincpu_rom.map subcpu_ram.bin subcpu_ram.map subcpu_rom.bin subcpu_rom.map) do (
  if exist "!OUTPUT_PATH!\%%i" del "!OUTPUT_PATH!\%%i"
)
goto end

:error
echo Build aborted.
exit /b 1

:end
