#ifndef __ADCHANDLER_H__
#define __ADCHANDLER_H__

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

#include <stdint.h>

// Helper function: this will either read buffered analog values, or use the ADC wait interrupt if ADC_SUPPORT_ADIR is undefined or 0.
uint8_t INPUT_GetAnalogInput (uint8_t idx);

// MAME currently does not support ADIR, so there's no way to check whether the ADC is ready, causing a hang if we were to wait for it.
// A workaround is supplied here, which samples the first 3 analog inputs on the 3 IRQ2s that happen per frame.
// This is analoguous to the implementation in the Out Run game code.
#ifndef ADC_SUPPORT_ADIR
#define ADC_SUPPORT_ADIR 0
#endif // !ADC_SUPPORT_ADIR

#if !ADC_SUPPORT_ADIR

// This implements ADC polling on IRQ2. On the original game, this happens for the lower 3 ADC inputs.
// Using it will override any previous handler on IRQ2.

// There are 6 analog inputs, but we have only 3 IRQ2's per frame. 
// Adding more will add a frame delay (but may be acceptable for multiplayer).
#define MAX_ANALOG_INPUTS 6
#define MAX_ANALOG_INPUTS_ON_IRQ 3 

// Limits the number of analog inputs that we update on interrupt.
// The default is 3, maximum is 6.
void ADC_SetMaxAnalogInputs (uint8_t numInputs);

// Installs the IRQ2 handler for deferred ADC reading. Chains to any previous handler.
void ADC_InstallHandler ();

// Reads buffered analog input. IRQ2 handler needs to be installed.
uint8_t ADC_ReadInputDeferred (uint8_t idx);

#endif // !ADC_SUPPORT_ADIR

#ifdef __cplusplus
} // extern "C"
#endif // __cplusplus

#endif // __ADCHANDLER_H__
