#ifndef __COMM_H__
#define __COMM_H__

#ifdef __cplusplus
extern "C" 
{
#endif // __cplusplus

//---------------------------------------------------------------------------------------------------------------------------------------------------------------------

// Forward declaration.
struct _COMM_CONTEXT;
typedef struct _COMM_CONTEXT COMM_CONTEXT;

// Set up initial COMM state and install IRQ handler.
// The pins not used for data transfer are set to the values passed in the initDigitalState parameter.
// Note that this can be called only once, but it's the user's responsibility to store the COMM_CONTEXT structure.
void COMM_Init (COMM_CONTEXT* pContext, uint8_t initDigitalState, bool enableReceive);

// Non-blocking; will try to buffer as much as possible and return.
uint8_t COMM_Send (COMM_CONTEXT* pContext, const uint8_t* pData, uint8_t len);
uint8_t COMM_Receive (COMM_CONTEXT* pContext, uint8_t* pData, uint8_t len);

// Enables receive (pull BUSY low).
// This is automatically done by COMM_Receive, but if you want to use COMM_ReceiveAvailable before actual reads, 
// you have to manually enable this.
void COMM_EnableReceive (COMM_CONTEXT* pContext, bool enableReceive);

// Returns the number of bytes available for immediate receive.
// Please note that due to locking, hammering this function will slow down or even block transmissions.
uint8_t COMM_ReceiveAvailable (COMM_CONTEXT* pContext);

// Queries the number of buffered and free outgoing bytes.
// Please note that due to locking, hammering this function will slow down or even block transmissions.
void COMM_GetSendBufferState (COMM_CONTEXT* pContext, uint8_t* pSendBuffered, uint8_t* pSendFree);

//---------------------------------------------------------------------------------------------------------------------------------------------------------------------

// Internals (but needed to be able to place in user memory)
#include "comm_internal.h"

//---------------------------------------------------------------------------------------------------------------------------------------------------------------------

#ifdef __cplusplus
} // extern "C"
#endif // __cplusplus

#endif // __COMM_H__
