#ifndef __COMM_INTERNAL_H__
#define __COMM_INTERNAL_H__

#ifndef __COMM_H__
#error "Don't include directly!"
#endif // __COMM_H__

// Note: this also means we can reuse the 'external "C"' logic.

typedef struct 
{
	uint8_t* Buffer, Size;
	uint8_t  Usage, ReadPos, WritePos;
} RING_BUFFER_INFO;

// State internals. Please do not modify yourself.
struct _COMM_CONTEXT
{
	uint8_t           DigitalOutShadow;   // Shadow value for digital output register.
	uint8_t           OutBuffer[16],      // Send/receive buffers.
	                  InBuffer[16];
	RING_BUFFER_INFO  OutBufferInfo,      // Send/receive buffer pointers.
	                  InBufferInfo;
	bool              ReceiveEnabled;     // As enabled by the user.

	struct COMM_STATE
	{
		bool DataAvailable : 1;           // Data present in OutBuffer. Fixme: Could remove this.

		// Transition states. Combine with DataAvailable and AllowReceive for the full state.
		enum __attribute ((packed)) _STATE
		{
			// Starting states.
			kState_Idle            = 0,   // Nothing to send, receive disabled.
			kState_Ready           = 1,   // Receive enabled and buffer not full, nothing available.
			kState_DataAvail       = 2,   // Data available, receive disabled.
			kState_Ready_DataAvail = 3,   // Data available, receive enabled and buffer not full.

			// Transactional states.
			kState_StrobeWait,            // Waiting for nSTROBE to go high again.
			kState_NibbleWait1,           // Waiting for HostBusy to go high again.
			kState_NibbleWait2,           // Waiting for HostBusy to go low again (second nibble).
			kState_NibbleWait3,           // Waiting for HostBusy to go high again.
		} State;
	} DispatchState;

	IRQFUNC* PrevIRQHandler;              // Chained previous IRQ handler.
};

#endif // __COMM_INTERNAL_H__
