#include <io.h>
#include "text.h"
#include "hwinit.h"
#include "palette.h"
#include <tile.h>
#include <road.h>
#include <sprite.h>

void HW_Init (uint16_t flags, PackedColor bgcolor)
{
	// Initialize PPI in/output ports. Should be done before writing to them.
	// This mutes sound and disables the screen.
	if (flags & HWINIT_Reset_PPI)
	{
		PPI_Init ();
	}
	
	// Clear text ram to spaces.
	if (flags & HWINIT_Clear_Text)
	{
		TEXT_ClearTextRAM ();
		WATCHDOG_Reset ();
	}
	
	// Reset a single tile ram page, and switch to it.
	if (flags & HWINIT_Reset_Tiles)
	{
		TILE_Reset ();
		WATCHDOG_Reset ();
	}
	
	// Full sprite init.
	// Requires IRQ2 and 4 running.
	if (flags & HWINIT_Reset_Sprites)
	{
		SPRITE_InitFull ();
		WATCHDOG_Reset ();
	}
	
	// Clear road palette. This isn't really needed anymore, unless CPU1 is modifying the road patterns.
	if (flags & HWINIT_Clear_Road_Palette)
	{
		ROAD_ResetPalette (bgcolor);
		WATCHDOG_Reset ();
	}

	// Clear road patterns. Can be done from both CPUs.
	if (flags & HWINIT_Reset_Road)
	{
		ROAD_Reset ();
		PALETTE_SetColor (PALETTE_ROAD_SOLID_START_IDX, bgcolor); // Also set background color #0.
		WATCHDOG_Reset ();
	}

	// If we want a working screen, enable it.	
	if (flags & HWINIT_EnableScreen)
		PPI_EnableScreen (true);
	
	// Sound too (actually, there's still the AMP mute on the output pins)
	if (flags & HWINIT_EnableSound)
		PPI_EnableSound (true);
}
