#include "menu.h"
#include "text.h"

// Todo: hide item.
// 4 states.
static void MENU_DrawItem (const MenuState* pMenuState, uint8_t itemIdx, bool isSelected, bool isActive)
{
	TEXT_GotoXY (pMenuState->iLeft, pMenuState->iTop + (itemIdx * pMenuState->iItemSpacing));

	if (isSelected)
	{
		// Draw arrow.
		TEXT_SetColor (TEXT_Yellow); // We could blink this thing.
		TEXT_WriteRawChar (TEXTGLYPH_ARROWRIGHT);
	}
	else
	{
		// Remove arrow.
		TEXT_WriteChar (' ');
	}

	// Draw item text.
	TEXT_WriteChar (' ');
	TEXT_SetColor (isActive ? TEXT_Cyan : isSelected ? TEXT_White : TEXT_Gray);
	TEXT_Write (pMenuState->pItems[itemIdx].name);
}

void MENU_Init (MenuState* pMenuState, const MenuItem* pItems, uint8_t nItems, const MenuInitParams* pInitParams, uint8_t initFlags)
{
	if (pInitParams && (initFlags & MIF_POSITION))
	{
		pMenuState->iLeft = pInitParams->iLeft;
		pMenuState->iTop = pInitParams->iTop;
	}
	else
	{
		pMenuState->iLeft = 2;
		pMenuState->iTop = 2;
	}

	//pMenuState->iVisibleItems = (pInitParams && (initFlags & MIF_VISIBLEITEMS)) ? pInitParams->iVisibleItems : -1;
	pMenuState->iItemSpacing = (pInitParams && (initFlags & MIF_ITEMSPACING)) ? pInitParams->iItemSpacing : 2;
	pMenuState->iHighlightItem = (pInitParams && (initFlags & MIF_HIGHLIGHT)) ? pInitParams->iHighlightItem : -1;

	pMenuState->nItems = nItems;
	pMenuState->pItems = pItems;
	pMenuState->iSelection = 0;

	// Todo: cache item widths.
	
	// We should draw this thing, probably.
	for (uint8_t i=0; i<nItems; i++)
		MENU_DrawItem (pMenuState, i, i == pMenuState->iSelection, i == pMenuState->iHighlightItem);
}

void MENU_Update (MenuState* pMenuState, const InputState* pInputState)
{
	uint8_t selection = pMenuState->iSelection;
	
	if (INPUT_IsButtonPressed (pInputState, Button_Left))
	{
		if (selection == 0)
			selection = pMenuState->nItems;
		selection--;
	} 
	else if (INPUT_IsButtonPressed (pInputState, Button_Right))
	{
		selection++;
		if (selection == pMenuState->nItems)
			selection = 0;
	}
	
	if (selection != pMenuState->iSelection)
	{
		// Remove old selection (and arrow)
		if (pMenuState->iSelection != (uint8_t)-1)
			MENU_DrawItem (pMenuState, pMenuState->iSelection, false, pMenuState->iSelection == pMenuState->iHighlightItem);
		
		// Draw new selection.
		pMenuState->iSelection = selection;
		
		MENU_DrawItem (pMenuState, pMenuState->iSelection, true, pMenuState->iSelection == pMenuState->iHighlightItem);
		return;
	}
	
	// Handle item activation. Actually, handle several buttons here and let the callback choose what to do.
	static const InputButton handleButtons[] = { Button_Accelerate, Button_Brake, Button_Start };
	for (uint8_t i=0; i<sizeof(handleButtons)/sizeof(InputButton); i++)
	{
		InputButton button = handleButtons[i];
		
		// Base whether we use key repeat on whether this is an item that can be activated (special highlight).
		// If it can and is active, only handle the key press once (for example: playing music)
		bool isActiveItem = pMenuState->iHighlightItem != (uint8_t)-1 && pMenuState->iHighlightItem == pMenuState->iSelection;
		
		if (isActiveItem ? INPUT_IsButtonPressedNoRepeat (pInputState, button) : INPUT_IsButtonPressed (pInputState, button))
		{
			const MenuItem* pItem = &pMenuState->pItems[pMenuState->iSelection];
			pItem->pCallback (button, pItem->param);
		}
	}
}

void MENU_SetHighlightItem (MenuState* pMenuState, uint8_t iHighlightItem)
{
	if (pMenuState->iHighlightItem == iHighlightItem)
		return;
	
	if (pMenuState->iHighlightItem != (uint8_t)-1)
	{
		// Remove prior highlight.
		MENU_DrawItem (pMenuState, pMenuState->iHighlightItem, false, false);
	}

	// Draw new highlight.
	if (iHighlightItem != (uint8_t)-1)
		MENU_DrawItem (pMenuState, iHighlightItem, true, true);

	pMenuState->iHighlightItem = iHighlightItem;
}
