#ifndef __MENU_H__
#define __MENU_H__

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

#include <stdint.h>
#include <stdbool.h>
#include "input.h"

typedef void (MenuCallback (InputButton button, void* param));

typedef struct
{
	const char* name;
	MenuCallback* pCallback;
	void* param;
} MenuItem;

#define MIF_POSITION     0x1
#define MIF_ITEMSPACING  0x2
//#define MIF_VISIBLEITEMS 0x4
#define MIF_HIGHLIGHT    0x8

typedef struct
{
	uint8_t iLeft;
	uint8_t iTop;
	uint8_t iItemSpacing; // 1 or 2.
	//uint8_t iVisibleItems;
	uint8_t iHighlightItem;
} MenuInitParams;

typedef struct
{
	uint8_t nItems;
	const MenuItem* pItems;
	uint8_t iSelection;
	uint8_t iHighlightItem; // Single item that draws in a different color.
	uint8_t iLeft;
	uint8_t iTop;
	uint8_t iItemSpacing; // 1 or 2.
	//uint8_t iVisibleItems;
} MenuState;

// Initialize and draw the initial menu state.
// All members of pMenuState will be initialized. pInitParams is optional.
void MENU_Init (MenuState* pMenuState, const MenuItem* pItems, uint8_t nItems, const MenuInitParams* pInitParams, uint8_t initFlags);

// Update the menu state, draw changes, and handle item callbacks.
void MENU_Update (MenuState* pMenuState, const InputState* pInputState);

// Highlight a single item (cyan). -1 to disable highlight.
void MENU_SetHighlightItem (MenuState* pMenuState, uint8_t itemIdx);
inline uint8_t MENU_GetSelectedItem (const MenuState* pMenuState) { return pMenuState->iSelection; }
inline uint8_t MENU_GetItemPositionY (const MenuState* pMenuState, uint8_t itemIdx) { return pMenuState->iTop + pMenuState->iItemSpacing*itemIdx; }

#ifdef __cplusplus
} // extern "C"
#endif // __cplusplus

#endif // __MENU_H__
