#include <maincpu.h>
#include <stdint.h>
#include <stdbool.h>
#include "palette.h"

// Remove this for some minor speed gain.
#define SANITIZE_PALIDX(x) (x&0xfff)
//#define SANITIZE_PALIDX(x) (x)

void PALETTE_SetColorRGBH (uint16_t idx, uint8_t red, uint8_t green, uint8_t blue, bool shade)
{
	uint16_t color16 = 
		// Low bits go to bit 12-14
		((((uint16_t)red  )&1) << 12) |
		((((uint16_t)green)&1) << 13) |
		((((uint16_t)blue )&1) << 14) |
		// High 4 bits are stored 4:4:4
		((((uint16_t)red  )&0x1e) >> 1) |
		((((uint16_t)green)&0x1e) << 3) | 
		((((uint16_t)blue )&0x1e) << 7) |
		// Shade bit.
		(shade ? 0x8000 : 0);
		
	PALETTE_RAM_BASE[SANITIZE_PALIDX(idx)] = color16;
}

void PALETTE_SetColorRGB (uint16_t idx, uint8_t red, uint8_t green, uint8_t blue)
{
	PALETTE_SetColorRGBH (idx, red, green, blue, 0);
}

void PALETTE_SetColor16 (uint16_t idx, uint16_t color16)
{
	PALETTE_RAM_BASE[SANITIZE_PALIDX(idx)] = color16;
}

uint16_t PALETTE_GetColor16 (uint16_t idx)
{
	return PALETTE_RAM_BASE[SANITIZE_PALIDX(idx)];
}

// Does a generic 4:4:4 fadein from a single solid color. Can be used to fade aprites/tiles in from black, white, etc.
// Could have one more bit, but would need bit reordering.
void PALETTE_FadeFromSolid (uint16_t destIdx, const PackedColor* pSrcData, uint16_t nColors, const PackedColor solidColor, uint8_t progress)
{
	// Actually, we could just build a premul table since the source inuts are only 4 bits.
	// Progress * solid can also be precalculated. This is in 4:8 bits.
	uint16_t invProgress = 256-progress;
	uint16_t solidR =  (solidColor.Packed & 0xf)         * invProgress;
	uint16_t solidG = ((solidColor.Packed & 0xf0))       * invProgress;
	uint16_t solidB = ((solidColor.Packed & 0xf00) >> 8) * invProgress;
	
	for (uint16_t idx = 0; idx<nColors; idx++)
	{
		uint16_t srcColor = (*pSrcData++).Packed;
		uint16_t r   =   (((srcColor & 0xf) * progress) + solidR) >> 8;
		uint16_t g   =  ((((srcColor & 0xf0) * progress) + solidG) >> 8) & 0xf0;
		uint16_t b   = (((((srcColor & 0xf00) >> 8) * progress) + solidB)) & 0xf00;
		PALETTE_SetColor16 (destIdx++, r|g|b);
	}
}
