#include "hwinit.h"
#include "text.h"
#include "tile.h"
#include "palette.h"
#include "tileunpack.h"
#include <stdint.h>
#include "shared.h"
#include <irq.h>
#include "input.h"

extern const TileGraphics CloudGraphics;
extern const TileGraphics HillGraphics;

void ROAD_DefaultPalette ()
{
	// Road colors.
	static const PackedColor roadPalette[] = 
	{
//		0x7999, 0x0999, 0x7999, 0x0fff, 0x0fff, 0x0999, 0x7999, 0x0999,
//		0x7999, 0x0999, 0x7999, 0x0fff, 0x0fff, 0x0999, 0x7999, 0x0999,
		0x0a88, 0x7a88, 0x0a88, 0x0fff, 0x0fff, 0x000a, 0x0a88, 0x7a88,
		0x0a88, 0x7a88, 0x0a88, 0x0fff, 0x0fff, 0x000a, 0x0a88, 0x7a88,
	};

	// Taken from the first level.
	static const PackedColor roadBackground[] = 
	{
//		0x089a, 0x789a, 0x089a, 0x789a, 0x09ab, 0x79ab, 0x0abc, 0x7abc,
//		0x0bcd, 0x7bcd, 0x0cde, 0x7cde, 0x0cde, 0x7cde, 0x0cde, 0x7cde, 
//		0x089a, 0x789a, 0x089a, 0x789a, 0x09ab, 0x79ab, 0x0abc, 0x7abc,
//		0x0bcd, 0x7bcd, 0x0cde, 0x7cde, 0x0cde, 0x7cde, 0x0cde, 0x7cde, 
		0x0586, 0x7586, 0x0586, 0x7586, 0x0697, 0x7697, 0x07a8, 0x77a8,
		0x08b9, 0x78b9, 0x09ca, 0x79ca, 0x09ca, 0x79ca, 0x09ca, 0x79ca,
		0x0586, 0x7586, 0x0586, 0x7586, 0x0697, 0x7697, 0x07a8, 0x77a8,
		0x08b9, 0x78b9, 0x09ca, 0x79ca, 0x09ca, 0x79ca, 0x09ca, 0x79ca,
	};

	unsigned short i;
	for (i=PALETTE_ROAD_START_IDX; i<PALETTE_ROAD_END_IDX; i++)
		PALETTE_SetColor (i, roadPalette[i-PALETTE_ROAD_START_IDX]);
	for (i=PALETTE_ROAD_BG_START_IDX; i<PALETTE_ROAD_BG_END_IDX; i++)
		PALETTE_SetColor (i, roadBackground[i-PALETTE_ROAD_BG_START_IDX]);

	// Background colors (scanline fill mode).
	static const PackedColor FillColors[] = {
		0x46ef, 0x46ef, 0x27df, 0x47df, 0x28cf, 0x48cf, 0x29bf, 0x49bf,
		0x0abf, 0x4abf, 0x0bbf, 0x4bbf, 0x1cbe, 0x4cbe, 0x3ead, 0x4ead,
		0x3f9c, 0x0f9c, 0x3f8b, 0x0f8b, 0x3f7a, 0x0f7a, 0x3f69, 0x0f69,
		0x2f59, 0x0f59, 0x3f48, 0x0f48, 0x3f37, 0x0f37, 0x0f37, 0x0f37,
		0x0f37, 0x0f37, 0x0f37, 0x0f37, 0x0f37, 0x0f37, 0x0f37, 0x0f37,
		0x0f37, 0x0f37, 0x0f37, 0x0f37, 0x0f37, 0x0f37, 0x0f37, 0x0f37,
		0x0f37, 0x0f37, 0x0f37, 0x0f37, 0x0f37, 0x0f37, 0x0f37, 0x0f37,
		0x0f37, 0x0f37, 0x0f37, 0x0f37, 0x0f37, 0x0f37, 0x0f37, 0x0f37,
	};
	for (i=0; i<(sizeof(FillColors)/sizeof(PackedColor)); i++)
		PALETTE_SetColor (i+PALETTE_ROAD_SOLID_START_IDX, FillColors[i]);
}

void main ()
{
	HW_Init (HWINIT_Default, PACKED_COLOR(0xf37));
	TEXT_InitDefaultPalette ();

	UnpackTileGraphics (8, &CloudGraphics);
	UnpackTileGraphics (4, &HillGraphics);

	// Set Y offsets.
	TileRegisters.BackgroundMapPages.Pages = 0x009a;
	TileRegisters.BackgroundScrollY.Y = 17*8;

	TileRegisters.ForegroundMapPages.Pages = 0x0047;
	TileRegisters.ForegroundScrollY.Y = 17*8;
		
	// Initialize from CPU0.
	RoadTest_Init ();
	ROAD_DefaultPalette ();

	TEXT_GotoXY (15,1);
	TEXT_SetColor (TEXT_Yellow);
	TEXT_Write ("road sample");
	TEXT_GotoXY (10,3);
	TEXT_SetColor (TEXT_White);
	TEXT_Write ("Updating road on CPU");

	InputState inputState;
	INPUT_Init (&inputState, false);
	
	// Animate road from main (but paint from sub)
	for (;;)
	{
		IRQ4_Wait();
		
		// Animation always happens on CPU0, to demonstrate the .shared section.
		distance -= 8; // ~120km/h

		// Draw road from this CPU?		
		if (updateFromCPU == 0)
			RoadTest_Update ();
		
		// Shifter toggles between CPUs.
		INPUT_Update (&inputState);
		updateFromCPU = INPUT_IsButtonHeld (&inputState, Button_Gear) ? 1 : 0;
		TEXT_GotoXY (31,3);
		TEXT_WriteChar ('0'+updateFromCPU);
	}
}
