#!/bin/bash 

# This makes sure we exit if anything unexpected happens.
set -e

# Global settings.
OUTRUN_GCC_ARCH=m68k-elf
OUTRUN_GCC_PREFIX=$OUTRUN_GCC_ARCH-
OUTRUN_GCC_CPUTYPE=m68000
OUTRUN_SDK_INCLUDE=include

# Check for the SDK.
if [ -z "$OUTRUN_SDK_PATH" ]
then
  echo OUTRUN_SDK_PATH environment variable not set!
  exit 1
fi

# Check for the compiler.
# Set GCC version, if not defined externally.
if [ -z "$OUTRUN_GCC_VERSION" ]
then
  OUTRUN_GCC_VERSION=8.2.0
fi

if [ -z "$OUTRUN_GCC_PATH" ]
then
  if [ -e "$OUTRUN_SDK_PATH/../gcc-$OUTRUN_GCC_VERSION" ]
  then
    # try gcc-<version>
    OUTRUN_GCC_PATH=$(cd $OUTRUN_SDK_PATH/../gcc-$OUTRUN_GCC_VERSION; pwd)
    echo Using $OUTRUN_GCC_PATH
  else
    echo OUTRUN_GCC_PATH environment variable not set!
    exit 1
  fi
fi

outputbasedir=obj
if [ "$1" != "clean" ]; then
  echo Building the Out Run Arcade SDK
  echo GCC version: $OUTRUN_GCC_VERSION
  echo
  if ! [ -d "$outputbasedir" ]; then mkdir $outputbasedir; fi
fi

buildcpus="0 1"
for cpu in $buildcpus; do
  cpuinputdir=cpu$cpu
  outputdir=$outputbasedir/cpu$cpu
  cpudef=CPU$cpu

  # CPU specific
  # Clean out archive listing.
  if [ -e "$outputdir/lib.in" ]; then rm "$outputdir/lib.in"; fi

  # Make output path.
  if [ "$1" != "clean" ]; then
    if ! [ -d "$outputdir" ]; then mkdir $outputdir; fi
    echo Compiling for CPU $cpu:
  fi

  for file in src/*.c src/*.cpp src/*.s src/$cpuinputdir/*.c src/$cpuinputdir/*.cpp src/$cpuinputdir/*.s; 
  do
    if [ -e "$file" ]; then 

      filename=$(basename "$file")
      basename="${filename%.*}"
      extension="${filename##*.}"
      objname="$basename".o

      if [ "$1" == "clean" ]; then
        # Remove old obj file.
      if [ -e "$outputdir"/"$objname" ]; then rm "$outputdir/$objname"; fi
      else
        echo "  "$filename

        if [ $extension == "c" ]; then
          "$OUTRUN_GCC_PREFIX"gcc -c $file -std=gnu11 -"$OUTRUN_GCC_CPUTYPE" -o $outputdir/$objname -Os -D$cpudef -I$OUTRUN_SDK_INCLUDE -I$OUTRUN_SDK_INCLUDE/$cpuinputdir
        fi # c

        if [ $extension == "cpp" ]; then
          "$OUTRUN_GCC_PREFIX"g++ -c $file --no-rtti -"$OUTRUN_GCC_CPUTYPE" -o $outputdir/$objname -Os -D$cpudef --no-exceptions -I$OUTRUN_SDK_INCLUDE -I$OUTRUN_SDK_INCLUDE/$cpuinputdir
        fi # cpp

        if [ $extension == "s" ]; then
          "$OUTRUN_GCC_PREFIX"as $file -"$OUTRUN_GCC_CPUTYPE" -o $outputdir/$objname --defsym $cpudef=1
        fi # s
      fi

      # Append to library input list
      echo $outputdir/$objname >> "$outputdir/lib.in"
    fi
  done

  # Remove old libraries
  libname=outrun_sdk_cpu$cpu.lib
  if [ -e "lib/$libname" ]; then rm lib/$libname; fi

    if [ "$1" != "clean" ]; then

    if [ -e "$outputdir/lib.in" ]; then
      echo Creating library...
      if ! [ -d lib ]; then mkdir lib; fi
        "$OUTRUN_GCC_PREFIX"ar --target=elf32-m68k lib/$libname -c -r @$outputdir/lib.in
      fi
    fi

  if [ "$1" != "clean" ]; then echo; fi
done # for (CPU)
echo Done.
