#include <maincpu.h>
#include <stdint.h>
#include <irq.h>
#include <io.h>
#include "sprite.h"

// Minimal sprite init.
// The game rom clears both entire lists (alhough doesn't swap after the second),
// and pulls SG1 low and then sets it high again.
void SPRITE_InitFast ()
{
	PPI_Write_Register (PPI_WRITE_PORT_C, PPI_C_SG1, PPI_C_SG0|PPI_C_SG1|PPI_C_CONT);
	IRQ4_Wait(); // y=226
	*((volatile uint32_t*)SpriteList) = 0xc0000000; // Hide sprite and terminate list.
	__asm__("not.w 0x140070.l"); // Swap buffers.
//	IRQ4_Wait();
}

// Full sprite reset; empties both lists and pulses SG1 low.
void SPRITE_InitFull ()
{
	PPI_Write_Register (PPI_WRITE_PORT_C, PPI_C_SG1, PPI_C_SG0|PPI_C_SG1|PPI_C_CONT); 
	IRQ4_Wait();

	volatile uint32_t* dst = (volatile uint32_t*)SpriteList;
	for (uint16_t i=0; i<128; i++, dst+=4)
	{	
		dst[0] = 0xc0000000;
		dst[1] = 0;
		dst[2] = 0;
		dst[3] = 0;
	}

	// Flip list and wait.
	__asm__("not.w 0x140070.l");
	
	IRQ4_Wait();
	IRQ4_Wait();

	// Clear other half.
	dst = (volatile uint32_t*)SpriteList;
	for (uint16_t i=0; i<128; i++, dst+=4)
	{	
		dst[0] = 0xc0000000;
		dst[1] = 0;
		dst[2] = 0;
		dst[3] = 0;
	}
	
	// Clear, hold and reenable SG1
    IRQ2_Wait();
	PPI_Write_Register (PPI_WRITE_PORT_C, 0, PPI_C_SG1);
	IRQ4_Wait();
	IRQ2_Wait();
	PPI_Write_Register (PPI_WRITE_PORT_C, PPI_C_SG1, PPI_C_SG1);
}
