#include <irq.h>
#include "road.h"

#ifdef CPU0

#include "palette.h"

// Sets all colors possibly used by the road generator to the same color.
// Regardless of what is in the road patterns, the screen will be blank (but tiles/sprites may disappear).
void ROAD_ResetPalette (const PackedColor bgColor)
{
	for (uint16_t i=PALETTE_ROAD_START_IDX; i<PALETTE_ROAD_END_IDX; i++)
		PALETTE_SetColor (i, bgColor);
	for (uint16_t i=PALETTE_ROAD_BG_START_IDX; i<PALETTE_ROAD_BG_END_IDX; i++)
		PALETTE_SetColor (i, bgColor);

	// Background colors (scanline fill mode).
	for (uint16_t i=0; i<128; i++)
		PALETTE_SetColor (i+PALETTE_ROAD_SOLID_START_IDX, bgColor);
}

#endif // CPU0

// Sets road priorities. From Mame's segaic16.c:
//   0 = road 0 only visible
//   1 = both roads visible, road 0 has priority
//   2 = both roads visible, road 1 has priority
//   3 = road 1 only visible
void ROAD_SetPriority (unsigned char priority)
{
	priority &= 3;
	*ROAD_CONTROL = priority;
}

// Schedules a buffer swap, which will occur on scanline 0. This will swap the RAM
// that is drawn by the road hardware with the RAM that is being written to at SUB_ROAD_BASE.
// Somewhat careful timing is needed, which Mame doesn't emulate properly.
// Since we only have an interrupt on the VBLANK at scanline 223, we should schedule the flip
// and finish writing the RAM contents before line 262/0, or delay everything until that line.
void ROAD_ScheduleSwapBuffers ()
{
	volatile uint8_t forceRead = *ROAD_CONTROL; // Read (this isn't optimized away due to 'volatile' on ROAD_CONTROL)
//#ifdef CPU0
//	asm("tst.b 0x290001");
//#else
//	asm("tst.b 0x90001");
//#endif
}

// Safe swap for initialization code. Could take two frames but is safe to call at any time.
void ROAD_SwapBuffers ()
{
	// Wait for scanline 224 and schedule a flip at scanline 0.
	IRQ4_Wait ();
	ROAD_ScheduleSwapBuffers ();

	// Wait again for scanline 224.
	IRQ4_Wait ();
}

// Reset to a single background color (0x0, palette entry 0x780).
void ROAD_Reset ()
{
	// Road 0 only.
	ROAD_SetPriority (0);

	// Reset the pattern and scroll registers.
	IRQ4_Wait ();
	for (uint16_t y=0; y<ROAD_SCREEN_HEIGHT; y++)
	{
		ROAD_PATTERN_0[y] = ROAD_SOLID_COLOR(0);
		ROAD_SCROLL_0[y]  = ROAD_SCROLL_CENTER;
	}

	// And flip. Assumes we can do the above between scanline 224 and 262 (=0).
	ROAD_ScheduleSwapBuffers ();
}
