#ifndef __ERROR_H__
#define __ERROR_H__

#include "platform.h"
#include <io.h>
#include <stdio.h>

#if !defined(COMPILER_SUPPORT_VARIADIC_MACROS) && _MSC_VER >= 1400
#define COMPILER_SUPPORT_VARIADIC_MACROS 1
#endif

static inline void SetColor (uint8 color, FILE* f)
{
	if (!_isatty( _fileno( f )))
		return;
	HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);  // Get handle to standard output
	SetConsoleTextAttribute(hConsole, color);
}

// Could do colors and stderr here.
#if COMPILER_SUPPORT_VARIADIC_MACROS
#define printerr(fmt, ...)  { SetColor(12, stdout); fprintf (stdout, fmt, __VA_ARGS__); SetColor(7, stdout); }
#define printwarn(fmt, ...) { SetColor(14, stdout); fprintf (stdout, fmt, __VA_ARGS__); SetColor(7, stdout); }
#else
#define printerr printf
#define printwarn printf
#endif // !COMPILER_SUPPORT_VARIADIC_MACROS

extern bool verbose;
#define printverbose if(verbose) printf

#endif // __ERROR_H__
